module  max2_i2c_slave_top(i2c_sda, i2c_scl, ilsns_rst, ilsns_data, led_r, led_g, led_b);

	// I2C
	inout			i2c_sda;
	input			i2c_scl;

	// Sensor
	output			ilsns_rst;
	input			ilsns_data;

	// LED
	output			led_r;
	output			led_g;
	output			led_b;

	////////////////////////////////////////////////
	// OSC (5.5MHz)
	////////////////////////////////////////////////
	wire	clk;

	max2_osc u0 (
		.oscena(1'b1),
		.osc(clk)
	);


	////////////////////////////////////////////////
	// PON reset
	////////////////////////////////////////////////
	wire		rst_n;
	reg [15:0]	rst_cnt_reg;

	always @(posedge clk) begin
		if (rst_cnt_reg != 16'hA55A)
			rst_cnt_reg	<= rst_cnt_reg + 1'b1;
	end

	assign	rst_n	= (rst_cnt_reg == 16'hA55A) ? 1'b1 : 1'b0;


	////////////////////////////////////////////////
	// i2c slave
	////////////////////////////////////////////////

	wire [7:0]	reg_addr, reg_data_out;
	reg [7:0]	reg_data_in;
	wire		reg_write, reg_read;

	i2c_slave_core u1 (
		.i2c_sda_in(i2c_sda),
		.i2c_sda_out(i2c_sda_out),
		.i2c_sda_oe(i2c_sda_oe),
		.i2c_scl_in(i2c_scl),

		.dev_addr(7'h12),

		.reg_addr_out(reg_addr),
		.reg_data_in(reg_data_in),
		.reg_data_out(reg_data_out),
		.reg_write_en_out(reg_write),
		.reg_read_en_out(reg_read),

		.rst_n(rst_n),
		.clk(clk)
	);

	assign	i2c_sda	= (i2c_sda_oe == 1'b0) ? 1'bZ : i2c_sda_out;


	always @(reg_read or reg_addr or illuminate_out_reg) begin
		if (reg_read == 1'b1) begin
			if (reg_addr == 8'h11) begin
				reg_data_in	<= illuminate_out_reg[7:0];
			end else if (reg_addr == 8'h12) begin
				reg_data_in	<= illuminate_out_reg[15:8];
			end else if (reg_addr == 8'h13) begin
				reg_data_in	<= {4'd0, illuminate_out_reg[19:16]};
			end else begin
				reg_data_in	<= 8'h00;
			end
		end else begin
			reg_data_in	<= 8'h00;
		end
	end


	////////////////////////////////////////////////
	// Sensor
	////////////////////////////////////////////////
	reg	[4:0]	sns_data_sync_reg;

	reg 	[19:0]	illuminate_cnt_reg;
	reg 	[19:0]	illuminate_out_reg;

	// synchronizer
	always @(posedge clk) begin
		if (rst_n == 1'b0) begin
			sns_data_sync_reg	<= 4'hF;
		end else begin
			sns_data_sync_reg[2:0]	<= {sns_data_sync_reg[1:0], ilsns_data};

			if (sns_data_sync_reg[2:1] == 2'b11)
				sns_data_sync_reg[3]	<= 1'b1;
			else if (sns_data_sync_reg[2:1] == 2'b00)
				sns_data_sync_reg[3]	<= 1'b0;

			sns_data_sync_reg[4]	<= sns_data_sync_reg[3];
		end
	end

	// freq counter
	always @(posedge clk) begin
		if (rst_n == 1'b0) begin
			illuminate_cnt_reg	<= 20'd0;
			illuminate_out_reg	<= 20'd0;
		end else begin
			if (sns_data_sync_reg[4:3] == 2'b01) begin			// rising edge: start count
				illuminate_cnt_reg	<= 20'd0;
			end else if (sns_data_sync_reg[4:3] == 2'b10) begin	// falling edge: end count
				illuminate_out_reg	<= illuminate_cnt_reg;
			end else if (sns_data_sync_reg[4] == 1'b1 && illuminate_cnt_reg != 20'b11111111111111111111) begin
				illuminate_cnt_reg	<= illuminate_cnt_reg + 1'b1;
			end
		end
	end

	assign	ilsns_rst	= ~rst_n;


	////////////////////////////////////////////////
	// LED
	////////////////////////////////////////////////
	reg [7:0]	led_cnt_reg;
	reg [7:0]	led_r_val_reg, led_g_val_reg, led_b_val_reg;

	always @(posedge clk) begin		// synchronizer
		if (rst_n == 1'b0) begin
			led_cnt_reg		<= 8'd0;

			led_r_val_reg	<= 8'd0;
			led_g_val_reg	<= 8'd0;
			led_b_val_reg	<= 8'd0;

		end else begin
			led_cnt_reg	<= led_cnt_reg + 1'b1;

			if (reg_write == 1'b1) begin
				if (reg_addr == 8'h01)
					led_r_val_reg	<= reg_data_out;

				if (reg_addr == 8'h02)
					led_g_val_reg	<= reg_data_out;

				if (reg_addr == 8'h03)
					led_b_val_reg	<= reg_data_out;
			end
		end
	end

	// PO
	assign	led_r	= (led_cnt_reg < led_r_val_reg) ? 1'b1 : 1'b0;
	assign	led_g	= (led_cnt_reg < led_g_val_reg) ? 1'b1 : 1'b0;
	assign	led_b	= (led_cnt_reg < led_b_val_reg) ? 1'b1 : 1'b0;

endmodule
